/* Name: main.c
 * Author: <insert your name here>
 * Copyright: <insert your copyright message here>
 * License: <insert your license reference here>
 */

#include <avr/io.h>
#include <util/delay.h>


#define LEFT_G1G2       _BV(PA0)
#define LEFT_G3G4       _BV(PA1)
#define RIGHT_G1G2       _BV(PA3)
#define RIGHT_G3G4       _BV(PA2)



// NG Serial stuff
#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 100 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTB
#define serial_direction DDRB
#define serial_pins PINB
#define serial_pin_in (1 << PB2)
#define serial_pin_out (1 << PB1)

void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
    //
    // read character into rxbyte on pins pin
    //    assumes line driver (inverts bits)
    //
    *rxbyte = 0;
    while (pin_test(*pins,pin))
        //
        // wait for start bit
        //
        ;
    //
    // delay to middle of first data bit
    //
    half_bit_delay();
    bit_delay();
    //
    // unrolled loop to read data bits
    //
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 0);
    else
        *rxbyte |= (0 << 0);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 1);
    else
        *rxbyte |= (0 << 1);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 2);
    else
        *rxbyte |= (0 << 2);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 3);
    else
        *rxbyte |= (0 << 3);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 4);
    else
        *rxbyte |= (0 << 4);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 5);
    else
        *rxbyte |= (0 << 5);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 6);
    else
        *rxbyte |= (0 << 6);
    bit_delay();
    if pin_test(*pins,pin)
        *rxbyte |= (1 << 7);
    else
        *rxbyte |= (0 << 7);
    //
    // wait for stop bit
    //
    bit_delay();
    half_bit_delay();
}

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
    //
    // send character in txchar on port pin
    //    assumes line driver (inverts bits)
    //
    // start bit
    //
    clear(*port,pin);
    bit_delay();
    //
    // unrolled loop to write data bits
    //
    if bit_test(txchar,0)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,1)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,2)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,3)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,4)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,5)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,6)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    if bit_test(txchar,7)
        set(*port,pin);
    else
        clear(*port,pin);
    bit_delay();
    //
    // stop bit
    //
    set(*port,pin);
    bit_delay();
    //
    // char delay
    //
    bit_delay();
}







void leftMotorOff(void)
{
    // Set both sides to 5v
    PORTA &= ~(LEFT_G1G2 | LEFT_G3G4);
}

void leftMotorForwards(void)
{
    PORTA &= ~(LEFT_G1G2 | LEFT_G3G4);
    PORTA |= LEFT_G1G2;
}

void leftMotorBackwards(void)
{
    PORTA &= ~(LEFT_G1G2 | LEFT_G3G4);
    PORTA |= LEFT_G3G4;
}

void rightMotorOff(void)
{
    // Set both sides to 5v
    PORTA &= ~(RIGHT_G1G2 | RIGHT_G3G4);
}

void rightMotorForwards(void)
{
    PORTA &= ~(RIGHT_G1G2 | RIGHT_G3G4);
    PORTA |= RIGHT_G1G2;
}

void rightMotorBackwards(void)
{
    PORTA &= ~(RIGHT_G1G2 | RIGHT_G3G4);
    PORTA |= RIGHT_G3G4;
}




int main(void)
{
    char chr;

    set(serial_port, serial_pin_out);
    output(serial_direction, serial_pin_out);

    // Set motor drive pins as outputs
    DDRA |= (LEFT_G1G2 | LEFT_G3G4 | RIGHT_G1G2 | RIGHT_G3G4 );
    
    leftMotorOff();
    rightMotorOff();
    
    for(;;)
    {
#if 0
        leftMotorOff();
        rightMotorOff();
        _delay_ms(1000);
        
        leftMotorForwards();
        rightMotorForwards();
        _delay_ms(1000);
        
        leftMotorBackwards();
        rightMotorBackwards();
        _delay_ms(1000);
#else
        get_char(&serial_pins, serial_pin_in, &chr);
        switch(chr)
        {
            case 'w':
                leftMotorForwards();
                break;
                
            case 's':
                leftMotorOff();
                break;
                
            case 'x':
                leftMotorBackwards();
                break;
            
            case 'o':
                rightMotorForwards();
                break;
            
            case 'k':
                rightMotorOff();
                break;
                
            case 'm':
                rightMotorBackwards();
                break;
                
            default:
                leftMotorOff();
                rightMotorOff();
                break;
        }
        
        // echo
        put_char(&serial_port, serial_pin_out, chr);
        //_delay_ms(1000);
        
#endif
        
    }
    return 0;
}
